#! /usr/bin/perl -w
## vh-fix.pl - Virtual Host fixup utility
## Copyright (C) 1999, 2000 The MadHouse Project
##
## This file is part of dep.pl.
##
## dep.pl is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## dep.pl is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
## $Id: vh-fix.pl.in,v 1.21 2000/12/12 15:54:39 algernon Exp $

require 5.004;

use POSIX qw(locale_h);
use locale;
use Getopt::Long;
use strict;
use sigtrap 'handler' => \&hup_trap, 'HUP';

require "getcwd.pl"; # I won't reinvent the wheel...

##########################################
# SETUP GLOBALS                          #
##########################################
my $confdir="/usr/local/etc/dep.pl";
my $userconfdir=$ENV{'HOME'}."/.dep.pl";
my $args=();
my $dirs=();
my $files=();
my $path_cache=();
my $VERSION=do {my @r=(q$Revision: 1.21 $=~/\d+/g); sprintf "%d."."%02d" x $#r, @r};
my $copyright="vh-fix.pl version $VERSION (C) 1999-2000 The MadHouse Project -";
my $banner="\n".$copyright."\n"."-"x(length($copyright))."\n\n";
my $version=$banner;
my $help="Usage: vh-fix.pl [OPTIONS] virtual-root output-file\n\n".
              "OPTIONS:\n".
	      "\t--follow,     -l : follow symlinks while installing libraries.\n".
	      "\t--force=[ln], -f : force use of the [ln] language.\n".
	      "\t--help,       -h : this help screen.\n".
	      "\t--noact,      -n : only output resulting script.\n".
	      "\t--nohup,      -N : ignore the HUP signal.\n".
	      "\t--quiet,      -q : quiet mode.\n".
	      "\t--version,    -v : print version information, then exit.\n\n";

my %messages=(
	"msg_running"	=> "Running %s ...",
	"msg_generate"  => "Generating %s ...",
	"msg_bye"	=> "",
	"msg_hup"	=> "Hanging up.",
	"vh_help"	=> $help
);

my %config=(
	"max_depth"	=> 100
);

# These are used later
my %files;
my %dirs;
my $cwd=&getcwd;

##########################################
# LOAD CONFIGURATION                     #
##########################################

$args->{"force"}="en";
loadconfig($confdir);

##########################################
# GET OPTIONS                            #
##########################################
Getopt::Long::config ("no_ignore_case");
Getopt::Long::config ("bundling");
Getopt::Long::config ("no_auto_abbrev");

# Remove the comment from the next line, if you want to...
#Getopt::Long::config ("pass_through");

my $optval = GetOptions (
	"help|h", \$args->{"help"},
	"version|v", \$args->{"version"},
	"force|f=s", \$args->{"force"},
	"noact|n", \$args->{"noact"},
	"quiet|q", \$args->{"quiet"},
	"follow|l", \$args->{"follow"},
	"nohup|N", \$args->{"nohup"},
);

$args->{"force"}=~s/=//g;

##########################################
# RELOAD CONFIGURATION                   #
##########################################

if ( $args->{"force"} ne "en" ) {
	loadconfig($confdir);
} else {
	undef ($args->{"force"});
}

loadconfig($userconfdir);

##########################################
# REACT ON DO-SOMETHING-THEN-EXIT PARAMS #
##########################################
# act on --version
if ($args->{"version"}) {
	print $version;
	exit;
}

# act on --help
if (@ARGV != 2 || $args->{"help"} || !$optval ) {
	print $banner;
	print $messages{"vh_help"};
	exit;
}

# if not quiet, display banner
if (!$args->{"quiet"}) {
	print $banner;
}

##########################################
# Run dep.pl                             #
##########################################

my $vroot=$ARGV[0];
$vroot=~s/\/$//g;

if ($vroot !~/^\//) {
	$vroot=resolve_path($cwd."/".$vroot);
}

my $params=" -qR -C=$vroot -x=$vroot/proc ";
if (defined ($args->{"nohup"}) ) {
	$params.="-n ";
}

if (!$args->{"quiet"}) {
	printf $messages{"msg_running"}."\n", "dep.pl".$params.$vroot;
}

my $tempfile=`tempfile`;
system ("dep.pl ".$params.$vroot." >$tempfile");

##########################################
# GENERATE INSTALL SCRIPT                #
##########################################

my $outfile=$ARGV[1];

if ($outfile !~/^\//) {
	$outfile=resolve_path($cwd."/".$outfile);
}

if (!$args->{"quiet"}) {
	printf $messages{"msg_generate"}."\n", $outfile;
}

open (INF,"<".$tempfile);
my @libs=<INF>;
close (INF);

my $lib;
my @tmp;
my $key;
foreach $lib (@libs) {
	chomp($lib);
	@tmp=split(/\//,$lib);
	$files{$lib}=1;
	$dirs{catd(@tmp)}=1;
}

open (OUTF, ">".$outfile);
print OUTF "#!/bin/sh\n";
print OUTF "#\n";
print OUTF "# Automatically generated by vh-fix.pl version $VERSION\n";
print OUTF "# (C) 1999-2000 The MadHouse Project\n#\n";
print OUTF "# ".localtime(time)."\n";
print OUTF "#\n\n";

print OUTF "# Install Directories\n";
foreach $key (keys %dirs) {
	print OUTF "install -d ".$vroot.$key."\n";
}

print OUTF "\n# Install shared libraries\n";
foreach $key (keys %files) {
	if (!defined ($args->{"follow"})) {
		print OUTF "install -m 0644 ".$key." ".$vroot.$key."\n";
	} else {
		my $paths;
		my $dest;
		my $pos;
		($paths, $dest)=target_of($key);
		print OUTF "install -m 0644 ".$dest." ".$vroot.$dest."\n";
		foreach $pos (keys %{$paths}) {
			print OUTF "ln -sf ".$paths->{$pos}." ".$vroot.$pos."\n";
			$pos=$paths->{$pos};
		}
	}
}

close (OUTF);
system ("rm -f $tempfile");
system ("chmod 755 $outfile");

##########################################
# RUN INSTALL SCRIPT                     #
##########################################

if ($outfile !~/\//) {
	$outfile="./".$outfile;
}

if (!$args->{"noact"}) {
	if (!$args->{"quiet"}) {
		printf $messages{"msg_running"}."\n", $outfile;
	}
	system ("$outfile");
}

if (!$args->{"quiet"}) {
	print "\n";
	print $messages{"msg_bye"}
}

##########################################
# SUBROUTINES                            #
##########################################
# catd (@)
sub catd {
	my @path=@_;
	return undef if !defined($#path);
	undef ($path[$#path]);
	my $element;
	my $fn;
	foreach $element (@path) {
		if (defined($element)) {
			$fn.=$element."/";
		}
	}
	return $fn;
}

# loadconfig ( [file] )
sub loadconfig {
	my $confdir=shift;
	my $line;
	my $language;
	my $prev;

	if ( !defined ($args->{"force"}) ) {
		$language="en";
	} else {
		$language=$args->{"force"};
	}

	if ( !$args->{"force"} && -d $confdir && -f $confdir."/language.conf" ) {
		open (INF,"<".$confdir."/language.conf");
		while (defined($line=<INF>)) {
			if ($line=~/^\s*language=\"?(..)\"?/i) {
				$language=$1;
			}
		}
		close (INF);
	}
	if ( -d $confdir && -f $confdir."/messages.".$language ) {
		open (INF, "<".$confdir."/messages.".$language);
		while (defined($line=<INF>)) {
			chomp($line);
			if ($line!~/^#/) {
				if ($line=~/^\s*([\w_]+)\s*=\s*\"?(.*?)(\"?)$/i) {
					$messages{"\L".$1}=$2;
					$prev=$1;
				}
				if ($line=~/^\s+\"?(.*?)(\"?)$/i && $prev) {
					$messages{"\L".$prev}.="\n".$1;
				}
			}
		}
		close (INF);
	}
}

# resolve_path ( [path] )
sub resolve_path {
	my $path=shift;
	my @pparts=split(/\//, $path);
	my $part;
	my $currpath;
	my $newpath;
	my $prevpath;
	foreach $part (@pparts) {
		$currpath.="/".$part;
		$currpath=~s/\/\//\//g;
		if ( defined ( $path_cache->{$currpath} ) ) {
			$newpath=$path_cache->{$currpath};
		} else {
			$path_cache->{$currpath}=target_of($currpath);
			$newpath=$path_cache->{$currpath};
		}
		$currpath=$newpath;
	}
	return rewrite_path($currpath, $newpath);
}

# target_of ( [source] )
sub target_of {
	my $source=shift;
	my $dest=$source;
	my $spath;
	my $depth=0;
	while ( -l $dest && $depth < $config{"max_depth"} ) {
		$spath->{$dest}=readlink($dest);
		$dest=rewrite_path($dest, $spath->{$dest});
		$depth++;
	}
	return ($spath, $dest);
}

# rewrite_path ( [orig], [sym] )
sub rewrite_path {
	my $orig=shift;
	my $sym=shift;
	my $sympath=$orig;
	if ($sym=~/^\//) {
		$sympath="";
	}
	$sympath=~s/^(.*)\/.*$/$1/g;
	$sympath.="/".$sym;
	$sympath=~s/\/\.(?=\/)//g;
	$sympath=~s/\/\.$//g;
	$sympath=~s/(.+?\/)([\.\/]+)/parse_dir($1,$2)/eg;
	$sympath=~s/\/\//\//g;
	return $sympath;
}

# parse_dir ( [path], [parents] )
sub parse_dir {
	my $path=shift;
	my $parents=shift;
	$parents=~s/\///g;
	my $tocut=length($parents)/2;
	my $out;
	my $i;
	my @pdb=split(/\//, $path);
	for ($i=0; $i<=$#pdb-$tocut; $i++) {
		$out.=$pdb[$i]."/";
	}
	return $out;
}

# hup_trap ()
sub hup_trap {
	if (!defined($args->{"nohup"})) {
		print "\n\n".$messages{"msg_hup"}."\n\n";
		exit;
	}
}

__END__
